import 'package:bloc/bloc.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

import '../../../../../widget/conditional_wrapper.dart';
import '../../utilityWidgets/bloc_module.dart';
import '../../utilityWidgets/loadableHydratedBloc/loadable_hydrated_bloc_event.dart';
import '../bloc/loadable_state_bloc.dart';
import '../bloc/loadable_state_state.dart';
import '../loadable_state.dart';
import 'loadable_state_background_loading.dart';
import 'loadable_state_error_bar.dart';
import 'loadable_state_error_screen.dart';
import 'loadable_state_primary_loading.dart';

class LoadableStateConsumer<TController extends Bloc<LoadableHydratedBlocEvent<TState>, LoadableState<TState>>, TState> extends StatelessWidget {
  final Widget Function(TState state, bool loading) child;
  final bool wrapWithScrollView;
  const LoadableStateConsumer({required this.child, this.wrapWithScrollView = false, super.key});

  static Duration animationDuration = const Duration(milliseconds: 200);

  @override
  Widget build(BuildContext context) {
    var loadableState = context.watch<TController>().state;
    var childWidget = ConditionalWrapper(
      condition: loadableState.reFetch != null,
      wrapper: (child) => RefreshIndicator(
        onRefresh: () {
          if(loadableState.reFetch != null) loadableState.reFetch!();
          return Future.value();
        },
        child: ConditionalWrapper(
          condition: wrapWithScrollView,
          wrapper: (child) => SingleChildScrollView(
            physics: const AlwaysScrollableScrollPhysics(),
            child: child
          ),
          child: child,
        )
      ),
      child: SizedBox(
        height: MediaQuery.of(context).size.height,
        child: loadableState.showContent()
          ? child(loadableState.data, loadableState.isLoading)
          : const SizedBox.shrink(),
      ),
    );

    return BlocModule<LoadableStateBloc, LoadableStateState>(
      create: (context) => LoadableStateBloc(),
      child: (context, bloc, state) {
        bloc.reFetch = loadableState.reFetch;
        return Column(
          children: [
            LoadableStateErrorBar(visible: loadableState.showErrorBar(), message: loadableState.error?.message, lastUpdated: loadableState.lastFetch),
            Expanded(
              child: Stack(
                children: [
                  LoadableStatePrimaryLoading(visible: loadableState.showPrimaryLoading()),
                  LoadableStateBackgroundLoading(visible: loadableState.showBackgroundLoading()),
                  LoadableStateErrorScreen(visible: loadableState.showError(), message: loadableState.error?.message),

                  AnimatedOpacity(
                    opacity: loadableState.showContent() ? 1.0 : 0.0,
                    duration: animationDuration,
                    curve: Curves.easeInOut,
                    child: childWidget,
                  ),
                ],
              ),
            )
          ],
        );
      }
    );
  }
}
