import 'dart:developer';

import 'package:flutter/material.dart';
import 'package:jiffy/jiffy.dart';

import '../../../../../view/pages/more/holidays/holidays.dart';
import '../../../../../widget/animatedTime.dart';
import '../../../../../widget/centeredLeading.dart';
import '../../../../../widget/debug/debugTile.dart';
import '../../../infrastructure/loadableState/loadable_state.dart';
import '../../../infrastructure/loadableState/view/loadable_state_consumer.dart';
import '../../../infrastructure/utilityWidgets/bloc_module.dart';
import '../bloc/holidays_bloc.dart';
import '../bloc/holidays_event.dart';
import '../bloc/holidays_state.dart';

class HolidaysView extends StatelessWidget {
  const HolidaysView({super.key});

  @override
  Widget build(BuildContext context) => BlocModule<HolidaysBloc, LoadableState<HolidaysState>>(
    create: (contet) => HolidaysBloc(),
    autoRebuild: true,
    child: (context, bloc, state) {
      log(state.toString());
      void showDisclaimer() {
        showDialog(context: context, builder: (context) => AlertDialog(
          title: const Text('Richtigkeit und Bereitstellung der Daten'),
          content: const Text(''
            'Sämtliche Datumsangaben sind ohne Gewähr.\n'
            'Ich übernehme weder Verantwortung für die Richtigkeit der Daten noch hafte ich für wirtschaftliche Schäden die aus der Verwendung dieser Daten entstehen können.\n\n'
            'Die Daten stammen von https://ferien-api.de/'),
          actions: [
            TextButton(child: const Text('Okay'), onPressed: () {
              bloc.add(DisclaimerDismissed());
              Navigator.of(context).pop();
            }),
          ],
        ));
      }
      
      return Scaffold(
        appBar: AppBar(
          title: const Text('Schulferien in Hessen'),
          actions: [
            IconButton(
              icon: const Icon(Icons.warning_amber_outlined),
              onPressed: showDisclaimer,
            ),
            PopupMenuButton<bool>(
              initialValue: bloc.showPastHolidays(),
              icon: const Icon(Icons.manage_history_outlined),
              itemBuilder: (context) => [true, false].map((e) => PopupMenuItem<bool>(
                  value: e,
                  enabled: e != bloc.showPastHolidays(),
                  child: Row(
                    children: [
                      Icon(e ? Icons.history_outlined : Icons.history_toggle_off_outlined, color: Theme.of(context).colorScheme.onSurface),
                      const SizedBox(width: 15),
                      Text(e ? 'Alle anzeigen' : 'Nur zukünftige anzeigen')
                    ],
                  )
              )).toList(),
              onSelected: (e) => bloc.add(SetPastHolidaysVisible(e)),
            ),
          ],
        ),
        body: LoadableStateConsumer<HolidaysBloc, HolidaysState>(
          child: (state, loading) => ListView.builder(
            itemCount: state.holidays.length,
            itemBuilder: (context, index) {
              var holiday = state.holidays[index];
              var holidayType = holiday.name.split(' ').first.capitalize();

              String formatDate(String enDate) => Jiffy.parse(enDate).format(pattern: 'dd.MM.yyyy');
              
              return ListTile(
                leading: const CenteredLeading(Icon(Icons.calendar_month)),
                title: Text('$holidayType ab ${formatDate(holiday.start)}'),
                subtitle: Text('bis ${formatDate(holiday.end)}'),
                onTap: () => showDialog(context: context, builder: (context) => SimpleDialog(
                  title: Text('$holidayType ${holiday.year} in Hessen'),
                  children: [
                    ListTile(
                      leading: const CenteredLeading(Icon(Icons.signpost_outlined)),
                      title: Text(holiday.name),
                      subtitle: Text(holiday.slug),
                    ),
                    ListTile(
                      leading: const Icon(Icons.arrow_forward),
                      title: Text('vom ${formatDate(holiday.start)}'),
                    ),
                    ListTile(
                      leading: const Icon(Icons.arrow_back),
                      title: Text('bis zum ${formatDate(holiday.end)}'),
                    ),
                    Visibility(
                      visible: !DateTime.parse(holiday.start).difference(DateTime.now()).isNegative,
                      replacement: ListTile(
                        leading: const CenteredLeading(Icon(Icons.content_paste_search_outlined)),
                        title: Text(Jiffy.parse(holiday.start).fromNow()),
                      ),
                      child: ListTile(
                        leading: const CenteredLeading(Icon(Icons.timer_outlined)),
                        title: AnimatedTime(callback: () => DateTime.parse(holiday.start).difference(DateTime.now())),
                        subtitle: Text(Jiffy.parse(holiday.start).fromNow()),
                      ),
                    ),
                    DebugTile(context).jsonData(holiday.toJson()),
                  ],
                )),
                trailing: const Icon(Icons.arrow_right),
              );
            },
          ),
        ),
      );
    },
  );
}
