import 'package:flutter/material.dart';

import 'marianum_message_view.dart';
import '../../../infrastructure/loadableState/loadable_state.dart';
import '../../../infrastructure/loadableState/view/loadable_state_consumer.dart';
import '../../../infrastructure/utilityWidgets/bloc_module.dart';
import '../bloc/marianum_message_bloc.dart';
import '../bloc/marianum_message_state.dart';

class MarianumMessageListView extends StatelessWidget {
  const MarianumMessageListView({super.key});

  @override
  Widget build(BuildContext context) => BlocModule<MarianumMessageBloc, LoadableState<MarianumMessageState>>(
    create: (context) => MarianumMessageBloc(),
    child: (context, bloc, state) => Scaffold(
        appBar: AppBar(
          title: const Text('Marianum Message'),
        ),
        body: LoadableStateConsumer<MarianumMessageBloc, MarianumMessageState>(
          child: (state, loading) => ListView.builder(
            itemCount: state.messageList.messages.length,
            itemBuilder: (context, index) {
              var message = state.messageList.messages.toList()[index];
              return ListTile(
                leading: const Column(
                  mainAxisAlignment: MainAxisAlignment.center,
                  children: [Icon(Icons.newspaper)],
                ),
                title: Text(message.name, overflow: TextOverflow.ellipsis),
                subtitle: Text('vom ${message.date}'),
                trailing: const Icon(Icons.arrow_right),
                onTap: () {
                  Navigator.push(context, MaterialPageRoute(builder: (context) => MessageView(basePath: state.messageList.base, message: message)));
                },
              );
            }
          ),
        ),
      )
  );
}
